var landscape = require('roboblogger-particle-assembler/assemblers/rbb-landscape');
var score = require('./score').score;
var subScore = require('./subScore');
var H = require('highland');
var R = require ('ramda');

exports.detect = function (events, rules, publishEvents, config) {
    return exports.makeDetector(events, rules, publishEvents, config).publishes;
};

exports.makeDetector = function (events, rules, publishEvents, config) {
    var detector = {
	events_and_publishes: H([
	    exports.ruleSetUpEvents(rules),
	    exports.insertPublishEvents(events, publishEvents)
	]).sequence()
    };
    detector.landscapes = exports.landscapes(
	detector.events_and_publishes,
	config
    );
    detector.scores = exports.scores(detector.landscapes, rules)
        .doto(function (x) { detector.last_score = x; });
    
    detector.publishes = detector.scores
        .filter(function (x) { return x.score.score === 1; })
        .reject(function (x) { return exports.isPublished(publishEvents, x); })
	.doto(function (x) { return exports.savePublish(publishEvents, x); });

    return detector;
};

var dedupePropertySet = function (x) {
    return new subScore.PropertySet(
	'$any',
	(x.rule.dedupe || []).map(function (pointer) {
	    return subScore.getPropertySet(x.landscape, pointer);
	})
    );
};

var propertySetToString = function (x) {
    if (x instanceof subScore.PropertySet) {
	return x.values.map(propertySetToString).join(',');
    }
    return JSON.stringify(x);
};

var dedupeValue = function (x) {
    return x.rule.id + ':' + propertySetToString(dedupePropertySet(x));
};

exports.isPublished = function (publishEvents, x) {
    var dedupe = dedupeValue(x);
    return R.find(function (y) { return y.data.dedupe == dedupe; })(publishEvents);
};

exports.savePublish = function (publishEvents, x) {
    publishEvents.push({
	data: {
	    type: {name: 'Publish'},
	    rule: x.rule,
	    dedupe: dedupeValue(x)
	}
    });
};

exports.transformPublishes = function (response) {
    return response.events.reduce(function (publishEvents, x) {
        exports.savePublish(publishEvents, x.event);
        return publishEvents;
    }, []);
};

exports.ruleSetUpEvents = function (rules) {
    return rules.map(function (rule) {
	return {
	    data: {
		type: {name: 'Rule set up'},
		rule: rule
	    }
	};
    });
}

// merges publish events into the event stream. only updates on new
// opta events (but past publishes will then be processed first).
exports.insertPublishEvents = function (events, publishEvents) {
    var i = 0;
    return events.consume(function (err, val, push, next) {
	// publish events get priority over all other events and are
	// immediately inserted into the stream. This is to ensure consistent
	// ordering of events
	while (i < publishEvents.length) {
	    push(null, publishEvents[i]);
	    i++;
	}
	if (err) {
	    push(err);
	    next();
	}
	else if (val === H.nil) {
	    push(null, val);
	}
	else {
	    push(null, val);
	    next();
	}
    });
};

exports.scores = function (landscapes, rules) {
    return landscapes.flatMap(function (landscape) {
        return H(rules).map(function (rule) {
	    return {
		score: score(landscape.context.particle, rule.rule)[0],
		event: landscape.event.data,
		index: landscape.event.index,
		sequenceId: landscape.event.head,
		landscape: R.clone ( landscape.context.particle ),
		rule: rule
	    };
        });
    });
};

exports.landscapes = function (events, config) {
    var initial = {
        context: landscape.initial(config),
        event: {}
    };
    return events.scan(initial, function (prev, event) {
	return {
            context: landscape.reducer(prev.context, event.data),
            event: event
	};
    });
};
