var detector = require('../detector');
var eventStream = require('../event-stream');
var utils = require('../utils');

var rules = [
    {
	"name": "2477041c-c556-49df-a0af-7b59e4fc14ed",
	"status": "published",
	"title": "Touches percentages by thirds",
	"type": "rbb-touchmap",
	"version": "0.2.4",
	"dedupe": [],
	"particle": {
	    "config": {
		"render": {
		    "publish-target": [
			"live-blog",
			"clock-watch"
		    ],
		    "graphicTemplate": "both-teams-thirds-pct",
		    "titleTemplate": "touch-thirds-fallback",
		    "textTemplate": ""
		}
	    }
	},
	"rule": {
	    "requires": "$all",
	    "subRules": [
		{
		    "threshold": 20,
		    "comparator": ">",
		    "pointer": [
			"match",
			"playedMinutes"
		    ]
		},
		{
		    "requires": "$any",
		    "subRules": [
			{
			    "requires": "$all",
			    "comment": "Midfield battle",
			    "subRules": [
				{
				    "threshold": 0,
				    "comparator": "===",
				    "pointer": [
					"home",
					"score"
				    ]
				},
				{
				    "threshold": 0,
				    "comparator": "===",
				    "pointer": [
					"away",
					"score"
				    ]
				},
				{
				    "threshold": 45,
				    "comparator": ">=",
				    "pointer": [
					"match",
					"touches",
					"thirds",
					"home",
					"midfield",
					"percentage"
				    ]
				},
				{
				    "threshold": 45,
				    "comparator": ">=",
				    "pointer": [
					"match",
					"touches",
					"thirds",
					"away",
					"midfield",
					"percentage"
				    ]
				},
				{
				    "threshold": 5,
				    "comparator": "<",
				    "pointer": [
					"DIFFERENCE",
					[
					    "match",
					    "touches",
					    "thirds",
					    "home",
					    "midfield",
					    "percentage"
					],
					[
					    "match",
					    "touches",
					    "thirds",
					    "away",
					    "midfield",
					    "percentage"
					]
				    ]
				},
				{
				    "threshold": -5,
				    "comparator": ">",
				    "pointer": [
					"DIFFERENCE",
					[
					    "match",
					    "touches",
					    "thirds",
					    "home",
					    "midfield",
					    "percentage"
					],
					[
					    "match",
					    "touches",
					    "thirds",
					    "away",
					    "midfield",
					    "percentage"
					]
				    ]
				}
			    ],
			    "render": {
				"titleTemplate": "touch-thirds-midfield-battle",
				"textTemplate": "touch-thirds-midfield-battle"
			    }
			},
			{
			    "requires": "$any",
			    "comment": "HA-high or AD-high",
			    "subRules": [
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 35,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"attack",
						"percentage"
					    ]
					},
					{
					    "threshold": 40,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"defence",
						"percentage"
					    ]
					}
				    ]
				},
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 45,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"defence",
						"percentage"
					    ]
					},
					{
					    "threshold": 30,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"attack",
						"percentage"
					    ]
					}
				    ]
				}
			    ],
			    "render": {
				"titleTemplate": "touch-thirds-HAH-or-ADH",
				"textTemplate": "touch-thirds-HAH-or-ADH"
			    }
			},
			{
			    "requires": "$any",
			    "comment": "AA-high or HD-high",
			    "subRules": [
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 33,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"attack",
						"percentage"
					    ]
					},
					{
					    "threshold": 35,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"defence",
						"percentage"
					    ]
					}
				    ]
				},
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 40,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"defence",
						"percentage"
					    ]
					},
					{
					    "threshold": 28,
					    "comparator": ">",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"attack",
						"percentage"
					    ]
					}
				    ]
				}
			    ],
			    "render": {
				"titleTemplate": "touch-thirds-AAH-or-HDH",
				"textTemplate": "touch-thirds-AAH-or-HDH"
			    }
			},
			{
			    "requires": "$any",
			    "comment": "HA-low or AD-low",
			    "subRules": [
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 20,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"attack",
						"percentage"
					    ]
					},
					{
					    "threshold": 25,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"defence",
						"percentage"
					    ]
					}
				    ]
				},
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 20,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"defence",
						"percentage"
					    ]
					},
					{
					    "threshold": 25,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"attack",
						"percentage"
					    ]
					}
				    ]
				}
			    ],
			    "render": {
				"titleTemplate": "touch-thirds-HAL-or-ADL",
				"textTemplate": "touch-thirds-HAL-or-ADL"
			    }
			},
			{
			    "requires": "$any",
			    "comment": "AA-low or HD-low",
			    "subRules": [
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 17,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"attack",
						"percentage"
					    ]
					},
					{
					    "threshold": 25,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"defence",
						"percentage"
					    ]
					}
				    ]
				},
				{
				    "requires": "$all",
				    "subRules": [
					{
					    "threshold": 20,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"home",
						"defence",
						"percentage"
					    ]
					},
					{
					    "threshold": 22,
					    "comparator": "<",
					    "pointer": [
						"match",
						"touches",
						"thirds",
						"away",
						"attack",
						"percentage"
					    ]
					}
				    ]
				}
			    ],
			    "render": {
				"titleTemplate": "touch-thirds-AAL-or-HDL",
				"textTemplate": "touch-thirds-AAL-or-HDL"
			    }
			}
		    ]
		}
	    ]
	},
	"author": {
	    "email": "",
	    "name": "Admin",
	    "given_name": "",
	    "family_name": ""
	},
	"id": "2477041c-c556-49df-a0af-7b59e4fc14ed",
	"publication_date": 1481028814759,
	"last_modified_date": 1481043999652
    }
];
 
// this array would normally be fetched using utils.fetchPublishes
var publishes = [];

utils.init ({
    optaApiUrl: 'https://opta-api-staging.eip.telegraph.co.uk/',
    rulesApiUrl: 'http://particle-api-staging.eip.telegraph.co.uk/rules'
});

// Example OPTA URL: https://opta-api-staging.eip.telegraph.co.uk/events/8/2016/855231
// Example publishes URL: https://opta-api-staging.eip.telegraph.co.uk/publishes/8/2016/855231
// Example Rules URL: http://particle-api-staging.eip.telegraph.co.uk/rules
// Example Rules URL: http://particle-api-test.eip.telegraph.co.uk/rules


// utils.fetchRules(function (err, rules) {
//     if (err) {
//         throw err;
//     }

    // utils.fetchPublishes(8, 2016, 855231, function (err, publishes) {
    //     if (err) {
    //         throw err;
    //     }

        var events = utils.events(8, 2016, 855306);

        utils.fetchLandscapeConfig(8, 2016, 855306, function (err, config) {
            if (err) {
                throw err;
            }

            detector.detect(events, rules, publishes, config).each(function (result) {
                console.log('---');
                // console.log(JSON.stringify(result, null, 4));
		console.log(result);
            });
        });

    // });

// });
