// wrapper to allow export as AMD in browser
(function() {
    'use strict';

    var methods,
        PropertySet = function PropertySet(type, values) {
            this.type = type;
            this.values = values;
        },
        exports = {
            returnObject: {
                score: 0,
                properties: {
                    matches: {},
                    fails: {}
                }
            },
            PropertySet: PropertySet,
            getPropertySet: function(val, pointer /*, ismethod*/ ) {

                var i, len = pointer.length,
                    result, methodArgumentsArray = [],
                    subPointer, subprops, mapProperties = function(value) {
                        return exports.getPropertySet(value, subPointer);
                    };
                // console.log(26,val, pointer);

                // detect method in first member of pointer and apply method

                if (pointer[0] && Object.keys(methods).indexOf(pointer[0]) !== -1) {

                    // apply pointerMethod using rest of pointer members as arguments
                    for (i = 1; i < len; i++) {

                        // if pointer argument is array, take it as a pointer, otherwise a value.
                        methodArgumentsArray.push(
                            Array.isArray(pointer[i]) ?
                            exports.getPropertySet(val, pointer[i] /*, 1*/ ) :
                            pointer[i]
                        );
                    }
                    result = methods[pointer[0]].apply(this, methodArgumentsArray);

                    return new PropertySet('$any', [result]);
                } else {

                    // add reducer-type and side pointer where not present (transitional)
                    //  pointer = exports.hydratePointer(pointer, exports.sideToViewFrom);
                    // console.log(pointer, val);
                    for (i = 0, len = pointer.length; i < len; i++) {
                        var k = pointer[i];

                        // events[-1] => last event
                        if (Array.isArray(val) && typeof k === 'number' && k < 0) {
                            k = val.length + k;
                        }
                        if (k === '$any' || k === '$all') {
                            subprops = [];
                            if (Array.isArray(val)) {
                                subPointer = pointer.slice(i + 1);
                                subprops = val.map(mapProperties);
                            }
                            return new PropertySet(k, subprops);
                        } else if (k in val) {
                            val = val[k];
                        } else {
                            return new PropertySet('$any', []);
                        }
                    }

                    return new PropertySet('$any', [val]);
                }
            },
            pointerString: function(pointer) {

                // render pointer as a string to be used in proeprties output
                var string = '';
                if (pointer[0] && Object.keys(methods).indexOf(pointer[0]) !== -1) {
                    for (var i = 0; i < pointer.length; i++) {
                        if (Array.isArray(pointer[i])) {
                            string += (i ? '=' : '') + pointer[i].join('.');
                        } else {
                            string += (i ? '=' : '') + pointer[i];
                        }
                    }
                    return string;
                } else {
                    return pointer.join('.');
                }
            },
            compare: function(value, comparator, threshold) {

                switch (comparator) {
                    case '===':
                    case '==':
                        return (Array.isArray(value) && value.indexOf(threshold) !== -1) || (value === threshold) || (Array.isArray(threshold) && threshold.indexOf(value) !== -1) ? 1 : 0;
                    case '!==':
                    case '!=':
                        return (Array.isArray(value) && value.indexOf(threshold) === -1) || (Array.isArray(threshold) && threshold.indexOf(value) === -1) || (value !== threshold) ? 1 : 0;
                    case '>=':
                        return (value >= threshold) ? 1 : value / threshold;
                    case '<=':
                        return (value <= threshold) ? 1 : threshold / value;
                    case '>':
                        return (value > threshold) ? 1 : value / (threshold + 1);
                    case '<':
                        return (value < threshold) ? 1 : (threshold - 1) / value;
                    default:
                        return 0;
                }
            },
            compareProperty: function(prop, comparator, threshold) {
                if (prop instanceof PropertySet) {
                    var f = function(p) {

                        return exports.compareProperty(p, comparator, threshold);
                    };
                    return exports[prop.type](f, prop.values);
                }
                return {
                    score: exports.compare(prop, comparator, threshold),
                    properties: {
                        matches: {},
                        fails: {}
                    }
                };
            },
            $any: function(f, values) {
                var truncatedValues = [],
                    result = {
                        score: 0,
                        properties: {
                            matches: {},
                            fails: {}
                        }
                    };
                if (!values.length) {
                    return result;
                }
                // bomb-out when one of condition set is true
                values.every(function(value) {
                    var evaluated = f(value);
                    truncatedValues.push(evaluated);
                    return evaluated.score !== 1;
                });

                // return values.map(f).reduce(function(result, x) {
                return truncatedValues.reduce(function(result, x) {
                    if (x.score > result.score) {
                        result.score = x.score;
                    }
                    return result;
                }, result);
            },
            $all: function(f, values) {
                var truncatedValues = [],
                    result = {
                        score: 1,
                        properties: {
                            matches: {},
                            fails: {}
                        }
                    };
                if (!values.length) {
                    return result;
                }
                // bomb-out when one of condition set is false
                values.every(function(value) {
                    var evaluated = f(value);
                    truncatedValues.push(evaluated);
                    return evaluated.score === 1;
                });
                var returnObject =
                    values.map(f).reduce(function(result, x) {
                        if (x.score < result.score) {
                            result.score = x.score;
                        }
                        return result;
                    }, result);
                return returnObject;
            },
            init: function(context, rule) {

                // returnObject is a collector of returned properties and sub-context attributes
                // during subRules iteration
                exports.returnObject = {
                    score: 0,
                    properties: {
                        matches: {},
                        fails: {}
                    }
                };
                return exports.subScore(context, rule);
            },
            subScore: function(context, rule) {

                // iterate through subRules
                var subRulesMatch, attr;
                if (rule.requires === 1) {
                    rule.requires = '$any';
                }
                if (rule.pointer !== undefined) {
                    var val, x,
                        matches;

                    // when pointer set to boolean true, score is set to 1 (used for else conditions)
                    if (rule.pointer === true) {
                        matches = {
                            score: 1,
                            properties: {
                                matches: {},
                                fails: {}
                            }
                        };
                        exports.returnObject.properties.matches.default = true;
                    } else {
                        x = exports.getPropertySet(context, rule.pointer);
                        matches = exports.compareProperty(x, rule.comparator, rule.threshold);

                        // USEFUL LOGIC DEBUG  console.log(matches.score===1, JSON.stringify(rule.pointer), rule.comparator, rule.threshold);
                        val = x.values[0];
                        while (val instanceof PropertySet) {
                            val = val.values[0];
                        }

                        // build returned properties
                        exports.returnObject.properties[(matches.score === 1 || matches.score === true) ? 'matches' : 'fails'][exports.pointerString(rule.pointer)] = val;
                    }
                    if (rule.render && (matches.score === 1 || matches.score === true)) {
                        for (attr in rule.render) {
                            exports.returnObject[attr] = rule.render[attr];
                        }
                    }

                    // pass back any sub-context settings found in logic
                    for (attr in exports.returnObject) {
                        if (attr !== 'score' && attr !== 'properties') {
                            matches[attr] = exports.returnObject[attr];
                        }
                    }
                    matches.properties = exports.returnObject.properties;
                    return matches;
                } else if (rule.subRules && (rule.requires === '$all' || rule.requires === '$any')) {
                    subRulesMatch = exports[rule.requires](
                        exports.subScore.bind(null, context),
                        rule.subRules
                    );

                    // copy any subcontexts or targets set in render to the output
                    if (rule.render && (subRulesMatch.score === 1 || subRulesMatch.score === true)) {
                        for (attr in rule.render) {
                            exports.returnObject[attr] = rule.render[attr];
                        }
                    }
                    for (var k in subRulesMatch.properties.matches) {
                        exports.returnObject.properties.matches[k] = subRulesMatch.properties.matches[k];
                    }
                    for (k in subRulesMatch.properties.fails) {
                        exports.returnObject.properties.fails[k] = subRulesMatch.properties.fails[k];
                    }
                    // pass back any sub-context settings found in logic
                    for (attr in exports.returnObject) {
                        if (attr !== 'score' && attr !== 'properties') {
                            subRulesMatch[attr] = exports.returnObject[attr];
                        }
                    }
                    subRulesMatch.properties = exports.returnObject.properties;
                    return subRulesMatch;
                }
                return exports.returnObject;
            }
        };


    methods = require('./pointerMethods.js');
    module.exports = {
        subScore: exports.init,
        PropertySet: exports.PropertySet,
        getPropertySet: exports.getPropertySet
    };

})();
