'use strict';
var config=require('./config'),
RuleEngine = require(config.scorePath),
    assert = require('assert'),
    R = require('ramda');


// Use this to check for mutations. In strict mode, mutating a frozen
// object will throw an error and fail the test.

function deepFreeze(obj) {
    var propNames = Object.getOwnPropertyNames(obj);
    propNames.forEach(function(name) {
        var prop = obj[name];
        if (typeof prop === 'object' && prop !== null) {
            deepFreeze(prop);
        }
    });
    return Object.freeze(obj);
}

function merge(a, b) {
    return deepFreeze(R.merge(a, b));
}

suite('arguments', function() {

    test('rule argument is ARRAY', function() {
        var context = deepFreeze({
            'match-period': 6
        });
        var rule = deepFreeze([{
            threshold: 6,
            comparator: '===',
            pointer: [
                'match-period'
            ]
        }]);

        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'match-period': 6
                }
            },
            particle: {
                'match-period': 6
            }
        }]);
        context = merge(context, {
            'match-period': 3
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'match-period': 3
                },
                matches: {}
            },
            particle: {
                'match-period': 3
            }
        }]);
    });

});

// ======================================

//              COMPARATORS

// ======================================

suite('comparators', function() {


    // We should avoid adding/mutating properties on the module.
    deepFreeze(RuleEngine);
    test('simple rule === comparator', function() {
        var context = deepFreeze({
            'match-period': 6
        });
        var rule = deepFreeze({
            threshold: 6,
            comparator: '===',
            pointer: [
                'match-period'
            ]
        });

        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'match-period': 6
                }
            },
            particle: {
                'match-period': 6
            }
        }]);
        context = merge(context, {
            'match-period': 3
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'match-period': 3
                },
                matches: {}
            },
            particle: {
                'match-period': 3
            }
        }]);
    });

    test('simple rule == comparator', function() {
        var context = deepFreeze({
            'match-period': 6
        });
        var rule = deepFreeze({
            threshold: 6,
            comparator: '==',
            pointer: [
                'match-period'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'match-period': 6
                }
            },
            particle: {
                'match-period': 6
            }
        }]);
        context = merge(context, {
            'match-period': 3
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'match-period': 3
                },
                matches: {}
            },
            particle: {
                'match-period': 3
            }
        }]);
    });

    test('simple rule !== comparator', function() {
        var context = deepFreeze({
            'match-period': 6
        });
        var rule = deepFreeze({
            threshold: 6,
            comparator: '!==',
            pointer: [
                'match-period'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'match-period': 6
                },
                matches: {}
            },
            particle: {
                'match-period': 6
            }
        }]);
        context = merge(context, {
            'match-period': 3
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'match-period': 3
                }
            },
            particle: {
                'match-period': 3
            }
        }]);
    });

    test('simple rule != comparator', function() {
        var context = deepFreeze({
            'match-period': 6
        });
        var rule = deepFreeze({
            threshold: 6,
            comparator: '!=',
            pointer: [
                'match-period'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'match-period': 6
                },
                matches: {}
            },
            particle: {
                'match-period': 6
            }
        }]);
        context = merge(context, {
            'match-period': 3
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'match-period': 3
                }
            },
            particle: {
                'match-period': 3
            }
        }]);
    });

    test('simple rule >= comparator', function() {
        var context = deepFreeze({
            'goals': 6
        });
        var rule = deepFreeze({
            threshold: 5,
            comparator: '>=',
            pointer: [
                'goals'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 6
                }
            },
            particle: {
                'goals': 6
            }
        }]);
        context = merge(context, {
            'goals': 10
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 10
                }
            },
            particle: {
                'goals': 10
            }
        }]);
        context = merge(context, {
            'goals': 3
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.6,
            properties: {
                fails: {
                    goals: 3
                },
                matches: {}
            },
            particle: {
                'goals': 3
            }
        }]);
        context = merge(context, {
            'goals': 1
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.2,
            properties: {
                fails: {
                    goals: 1
                },
                matches: {}
            },
            particle: {
                'goals': 1
            }
        }]);
        context = merge(context, {
            'goals': 0
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    goals: 0
                },
                matches: {}
            },

            particle: {
                'goals': 0
            }
        }]);
    });

    test('simple rule <= comparator', function() {
        var context = deepFreeze({
            'goals': 10
        });
        var rule = deepFreeze({
            threshold: 5,
            comparator: '<=',
            pointer: [
                'goals'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.5,
            properties: {
                fails: {
                    goals: 10
                },
                matches: {}
            },
            particle: {
                'goals': 10
            }
        }]);
        context = merge(context, {
            'goals': 20
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.25,
            properties: {
                fails: {
                    goals: 20
                },
                matches: {}
            },
            particle: {
                'goals': 20
            }
        }]);
        context = merge(context, {
            'goals': 5
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 5
                }
            },
            particle: {
                'goals': 5
            }
        }]);
        context = merge(context, {
            'goals': 1
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 1
                }
            },
            particle: {
                'goals': 1
            }
        }]);
        context = merge(context, {
            'goals': 0
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 0
                }
            },
            particle: {
                'goals': 0
            }
        }]);
    });

    test('simple rule > comparator', function() {
        var context = deepFreeze({
            'goals': 5
        });
        var rule = deepFreeze({
            threshold: 4,
            comparator: '>',
            pointer: [
                'goals'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 5
                }
            },
            particle: {
                'goals': 5
            }
        }]);
        context = merge(context, {
            'goals': 10
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 10
                }
            },
            particle: {
                'goals': 10
            }
        }]);
        context = merge(context, {
            'goals': 4
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.8,
            properties: {
                fails: {
                    goals: 4
                },
                matches: {}
            },
            particle: {
                'goals': 4
            }
        }]);
        context = merge(context, {
            'goals': 3
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.6,
            properties: {
                fails: {
                    goals: 3
                },
                matches: {}
            },
            particle: {
                'goals': 3
            }
        }]);
        context = merge(context, {
            'goals': 1
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.2,
            properties: {
                fails: {
                    goals: 1
                },
                matches: {}
            },
            particle: {
                'goals': 1
            }
        }]);
        context = merge(context, {
            'goals': 0
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    goals: 0
                },
                matches: {}
            },
            particle: {
                'goals': 0
            }
        }]);
    });

    test('simple rule < comparator', function() {
        var context = deepFreeze({
            'goals': 10
        });
        var rule = deepFreeze({
            threshold: 6,
            comparator: '<',
            pointer: [
                'goals'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.5,
            properties: {
                fails: {
                    goals: 10
                },
                matches: {}
            },
            particle: {
                'goals': 10
            }
        }]);
        context = merge(context, {
            'goals': 20
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.25,
            properties: {
                fails: {
                    goals: 20
                },
                matches: {}
            },
            particle: {
                'goals': 20
            }
        }]);
        context = merge(context, {
            'goals': 5
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 5
                }
            },
            particle: {
                'goals': 5
            }
        }]);
        context = merge(context, {
            'goals': 1
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 1
                }
            },
            particle: {
                'goals': 1
            }
        }]);
        context = merge(context, {
            'goals': 0
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    goals: 0
                }
            },
            particle: {
                'goals': 0
            }
        }]);
    });
});

suite('pointers', function() {
    test('nested property', function() {
        var context = deepFreeze({
            counters: {
                goals: 5
            }
        });
        var rule = deepFreeze({
            threshold: 5,
            comparator: '==',
            pointer: [
                'counters',
                'goals'
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'counters.goals': 5
                }
            },
            particle: {
                counters: {
                    goals: 5
                }
            }
        }]);
        rule = merge(rule, {
            threshold: 10
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'counters.goals': 5
                },
                matches: {}
            },
            particle: {
                counters: {
                    goals: 5
                }
            }
        }]);
    });

    test('$all subRules', function() {
        var context = deepFreeze({
            'match-period': 6,
            'passes': [1, 2, 3]
        });
        var rule = deepFreeze({
            'requires': '$all',
            'subRules': [{
                'threshold': 6,
                'comparator': '>=',
                'pointer': ['passes', 'length']
            }, {
                'threshold': 6,
                'comparator': '===',
                'pointer': ['match-period']
            }]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.5,
            properties: {
                fails: {
                    'passes.length': 3
                },
                matches: {
                    'match-period': 6
                }
            },
            particle: {
                'match-period': 6,
                'passes': [1, 2, 3]
            }
        }]);
        context = merge(context, {
            passes: [1, 2, 3, 4, 5, 6]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'passes.length': 6,
                    'match-period': 6
                }
            },
            particle: {
                'match-period': 6,
                'passes': [1, 2, 3, 4, 5, 6]
            }
        }]);
        context = merge(context, {
            'match-period': 1
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'match-period': 1
                },
                matches: {
                    'passes.length': 6
                }
            },
            particle: {
                'match-period': 1,
                'passes': [1, 2, 3, 4, 5, 6]
            }
        }]);
    });

    test('$any subRules', function() {
        var context = deepFreeze({
            'match-period': 6,
            'passes': [1, 2, 3]
        });
        var rule = deepFreeze({
            'requires': '$any',
            'subRules': [{
                'threshold': 6,
                'comparator': '>=',
                'pointer': ['passes', 'length']
            }, {
                'threshold': 6,
                'comparator': '===',
                'pointer': ['match-period']
            }]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {
                    'passes.length': 3
                },
                matches: {
                    'match-period': 6
                }
            },
            particle: {
                'match-period': 6,
                'passes': [1, 2, 3]
            }
        }]);
        context = merge(context, {
            passes: [1, 2, 3, 4, 5, 6]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'passes.length': 6
                }
            },
            particle: {
                'match-period': 6,
                'passes': [1, 2, 3, 4, 5, 6]
            }
        }]);
        context = merge(context, {
            'match-period': 1
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {
                    
                },
                matches: {
                    'passes.length': 6
                }
            },
            particle: {
                'match-period': 1,
                'passes': [1, 2, 3, 4, 5, 6]
            }
        }]);
        context = merge(context, {
            passes: [1, 2, 3]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.5,
            properties: {
                fails: {
                    'match-period': 1,
                    'passes.length': 3
                },
                matches: {}
            },
            particle: {
                'match-period': 1,
                'passes': [1, 2, 3]
            }
        }]);
    });

    test('nested $any and $all subRules', function() {
        var context = deepFreeze({
            a: 1,
            b: 2,
            c: 3,
            d: 4
        });
        var rule = deepFreeze({
            'requires': '$any',
            'subRules': [{
                'requires': '$all',
                'subRules': [{
                    'threshold': 1,
                    'comparator': '==',
                    'pointer': ['a']
                }, {
                    'threshold': 2,
                    'comparator': '==',
                    'pointer': ['b']
                }]
            }, {
                'requires': '$all',
                'subRules': [{
                    'threshold': 10,
                    'comparator': '>=',
                    'pointer': ['c']
                }, {
                    'threshold': 10,
                    'comparator': '>=',
                    'pointer': ['d']
                }]
            }]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                matches: {
                    a: 1,
                    b: 2
                },
                fails: {
                   
                }
            },
            particle: {
                a: 1,
                b: 2,
                c: 3,
                d: 4
            }
        }]);
        context = merge(context, {
            a: 0
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0.3,
            properties: {
                fails: {
                    a: 0,
                    c: 3,
                    d: 4
                },
                matches: {
                    b: 2
                }
            },
            particle: {
                a: 0,
                b: 2,
                c: 3,
                d: 4
            }
        }]);
        context = merge(context, {
            c: 30,
            d: 40
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {
                    a: 0
                },
                matches: {
                    b: 2,
                    c: 30,
                    d: 40
                }
            },
            particle: {
                a: 0,
                b: 2,
                c: 30,
                d: 40
            }
        }]);
        context = merge(context, {
            c: 0,
            d: 0
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    a: 0,
                    c: 0,
                    d: 0
                },
                matches: {
                    b: 2
                }
            },
            particle: {
                a: 0,
                b: 2,
                c: 0,
                d: 0
            }
        }]);
    });

    test('$any as property pointer', function() {
        var context = deepFreeze({
            events: [{
                player: 123
            }, {
                player: 456
            }, {
                player: 789
            }]
        });
        var rule = deepFreeze({
            threshold: 456,
            comparator: '==',
            pointer: ['events', '$any', 'player']
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'events.$any.player': 123
                }
            },
            particle: {
                events: [{
                    player: 123
                }, {
                    player: 456
                }, {
                    player: 789
                }]
            }
        }]);
        rule = merge(rule, {
            threshold: 999
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'events.$any.player': 123
                },
                matches: {}
            },
            particle: {
                events: [{
                    player: 123
                }, {
                    player: 456
                }, {
                    player: 789
                }]
            }
        }]);
    });

    test('$all as property pointer', function() {
        var context = deepFreeze({
            events: [{
                player: 123
            }, {
                player: 456
            }, {
                player: 789
            }]
        });
        var rule = deepFreeze({
            threshold: 456,
            comparator: '==',
            pointer: ['events', '$all', 'player']
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'events.$all.player': 123
                },
                matches: {}
            },
            particle: {
                events: [{
                    player: 123
                }, {
                    player: 456
                }, {
                    player: 789
                }]
            }
        }]);
        context = merge(context, {
            events: [{
                player: 456
            }, {
                player: 456
            }, {
                player: 456
            }]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'events.$all.player': 456
                }
            },
            particle: {
                events: [{
                    player: 456
                }, {
                    player: 456
                }, {
                    player: 456
                }]
            }
        }]);
    });

    test('nested $any and $all in pointer', function() {
        var context = deepFreeze({
            events: [{
                player: 123,
                kicks: [{
                    type: 'throw-in'
                }, {
                    type: 'shot'
                }]
            }, {
                player: 456,
                kicks: [{
                    type: 'pass'
                }, {
                    type: 'shot'
                }]
            }]
        });
        // if all players made a shot
        var rule = deepFreeze({
            threshold: 'shot',
            comparator: '==',
            pointer: ['events', '$all', 'kicks', '$any', 'type']
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'events.$all.kicks.$any.type': 'throw-in'
                }
            },
            particle: {
                events: [{
                    player: 123,
                    kicks: [{
                        type: 'throw-in'
                    }, {
                        type: 'shot'
                    }]
                }, {
                    player: 456,
                    kicks: [{
                        type: 'pass'
                    }, {
                        type: 'shot'
                    }]
                }]
            }
        }]);
        context = merge(context, {
            events: [{
                player: 123,
                kicks: [{
                    type: 'throw-in'
                }, {
                    type: 'pass'
                }]
            }, {
                player: 456,
                kicks: [{
                    type: 'pass'
                }, {
                    type: 'shot'
                }]
            }]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'events.$all.kicks.$any.type': 'throw-in'
                },
                matches: {}
            },
            particle: {
                events: [{
                    player: 123,
                    kicks: [{
                        type: 'throw-in'
                    }, {
                        type: 'pass'
                    }]
                }, {
                    player: 456,
                    kicks: [{
                        type: 'pass'
                    }, {
                        type: 'shot'
                    }]
                }]
            }
        }]);
        context = merge(context, {
            events: [{
                player: 123,
                kicks: [{
                    type: 'throw-in'
                }, {
                    type: 'pass'
                }]
            }, {
                player: 456,
                kicks: [{
                    type: 'pass'
                }, {
                    type: 'pass'
                }]
            }]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'events.$all.kicks.$any.type': 'throw-in'
                },
                matches: {}
            },
            particle: {
                events: [{
                    player: 123,
                    kicks: [{
                        type: 'throw-in'
                    }, {
                        type: 'pass'
                    }]
                }, {
                    player: 456,
                    kicks: [{
                        type: 'pass'
                    }, {
                        type: 'pass'
                    }]
                }]
            }
        }]);
    });

    test('negative array index', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 4]
        });
        var rule = deepFreeze({
            threshold: 4,
            comparator: '==',
            pointer: ['events', -1]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'events.-1': 4
                }
            },
            particle: {
                events: [1, 2, 3, 4]
            }
        }]);
        context = merge(context, {
            events: [1, 2, 3, 4, 5]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'events.-1': 5
                },
                matches: {}
            },
            particle: {
                events: [1, 2, 3, 4, 5]
            }
        }]);
        rule = merge(rule, {
            pointer: ['events', -2]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'events.-2': 4
                }
            },
            particle: {
                events: [1, 2, 3, 4, 5]
            }
        }]);
    });



});

suite('methods', function() {
    test('pointer method PRODUCT', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 4]
        });
        var rule = deepFreeze({
            threshold: 12,
            comparator: '==',
            pointer: ['PRODUCT', ['events', -1], 3]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'PRODUCT=events.-1=3': 12
                }
            },
            particle: {
                events: [1, 2, 3, 4]
            }
        }]);

        rule = merge(rule, {
            pointer: ['PRODUCT', ['events', -1],
                ['events', 1]
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'PRODUCT=events.-1=events.1': 8
                },
                matches: {}
            },
            particle: {
                events: [1, 2, 3, 4]
            }
        }]);
    });
    test('pointer method QUOTIENT', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        var rule = deepFreeze({
            threshold: 6,
            comparator: '==',
            pointer: ['QUOTIENT', ['events', -1], 3]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'QUOTIENT=events.-1=3': 4
                },
                matches: {

                }
            },
            particle: {
                events: [1, 2, 3, 12]
            }
        }]);

        rule = merge(rule, {
            pointer: ['QUOTIENT', ['events', -1],
                ['events', 1]
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'QUOTIENT=events.-1=events.1': 6
                }
            },
            particle: {
                events: [1, 2, 3, 12]
            }
        }]);
    });
    test('pointer method SUM', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        var rule = deepFreeze({
            threshold: 18,
            comparator: '==',
            pointer: ['SUM', ['events', -1], 3]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'SUM=events.-1=3': 15
                },
                matches: {

                }
            },
            particle: {
                events: [1, 2, 3, 12]
            }
        }]);

        rule = merge(rule, {
            pointer: ['SUM', ['events']]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'SUM=events': 18
                }
            },
            particle: {
                events: [1, 2, 3, 12]
            }
        }]);
    });
    test('pointer method DIFFERENCE', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        var rule = deepFreeze({
            threshold: 10,
            comparator: '==',
            pointer: ['DIFFERENCE', ['events', -1], 3]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'DIFFERENCE=events.-1=3': 9
                },
                matches: {

                }
            },
            particle: {
                events: [1, 2, 3, 12]
            }
        }]);

        rule = merge(rule, {
            pointer: ['DIFFERENCE', ['events', -1],
                ['events', 1]
            ]
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'DIFFERENCE=events.-1=events.1': 10
                }
            },
            particle: {
                events: [1, 2, 3, 12]
            }
        }]);
    });
});

suite('outputs', function() {


    test('rule.id, particle-type, passed to returnedObject', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        var rule = deepFreeze({
            rule: {
                threshold: 10,
                comparator: '==',
                pointer: ['DIFFERENCE', ['events', -1], 3]
            },
            id: 'dsfsdfsdfs'
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'DIFFERENCE=events.-1=3': 9
                },
                matches: {

                }
            },
            particle: {
                events: [1, 2, 3, 12]
            },
            ruleId: 'dsfsdfsdfs'

        }]);

        context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        rule = deepFreeze({
            rule: {
                threshold: 10,
                comparator: '==',
                pointer: ['DIFFERENCE', ['events', -1], 3]
            },
            particle: {
                'particle-type': 'rbb-goals-and-passes'
            }

        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'DIFFERENCE=events.-1=3': 9
                },
                matches: {

                }
            },
            particle: {
                events: [1, 2, 3, 12]
            },

            'particle-type': 'rbb-goals-and-passes'

        }]);
    });



    test('particle.config.render attributes passed to returnedObject', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        var rule = deepFreeze({
            rule: {
                threshold: 10,
                comparator: '==',
                pointer: ['DIFFERENCE', ['events', -1], 3]
            },
            particle: {
                config: {
                    render: {
                        textTemplate: 'tetetete',
                        titleTemplate: 'fdgddgd'
                    }
                }
            }
        });
        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'DIFFERENCE=events.-1=3': 9
                },
                matches: {

                }
            },
            particle: {
                events: [1, 2, 3, 12]
            },
            textTemplate: 'tetetete',
            titleTemplate: 'fdgddgd'

        }]);
    });

    test('render attrs in logic passed to returnedObject', function() {
        var context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        var rule = deepFreeze({
            rule: {
                threshold: 9,
                comparator: '==',
                pointer: ['DIFFERENCE', ['events', -1], 3],
                render: {
                    textTemplate: 'tetetete',
                    titleTemplate: 'fdgddgd'
                }
            },
            particle: {
                config: {
                    render: {
                        graphicTemplate: 'goal'
                    }
                }
            }
        });


        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 1,
            properties: {
                fails: {},
                matches: {
                    'DIFFERENCE=events.-1=3': 9
                }
            },
            particle: {
                events: [1, 2, 3, 12]
            },
            textTemplate: 'tetetete',
            titleTemplate: 'fdgddgd',
            graphicTemplate: 'goal'

        }]);
        context = deepFreeze({
            events: [1, 2, 3, 12]
        });
        rule = deepFreeze({
            rule: {
                threshold: 8,
                comparator: '==',
                pointer: ['DIFFERENCE', ['events', -1], 3],
                render: {
                    textTemplate: 'tetetete',
                    titleTemplate: 'fdgddgd'
                }
            },
            particle: {
                config: {
                    render: {
                        graphicTemplate: 'goal'
                    }
                }
            }
        });


        assert.deepEqual(RuleEngine.score(context, rule), [{
            score: 0,
            properties: {
                fails: {
                    'DIFFERENCE=events.-1=3': 9
                },
                matches: {}
            },
            particle: {
                events: [1, 2, 3, 12]
            },

            graphicTemplate: 'goal'

        }]);
    });

});
