var url = require('url');
var eventStream = require('./event-stream');
var request = require('request');
var R = require ('ramda');
var H = require('highland');
var detector = require('./detector');


var config;
exports.init = (conf) => {
    config = conf;
};

exports.fetchRules = (callback) => {
    request.get({url: `${config.rulesApiUrl}/rules`, json: true, qs: {
        version: config.version,
        status: 'published'
    }}, (err, res) => {
	    return callback(err, res.body);
    });
};

exports.fetchPublishes = (competitionId, seasonId, fixtureId, callback) => {
    const pubs_path = ['publishEvents', competitionId, seasonId, fixtureId].join('/');
    const pubs_url = url.resolve(config.optaApiUrl, '/' + pubs_path);
    request.get({url: pubs_url, json: true}, (err, res) => {
        if (err) {
            return callback(err);
        }
        return callback(null, detector.transformPublishes(res.body));
    });
};

exports.postPublishes = (competitionId, seasonId, fixtureId, publishes, callback) => {
    const pubs_path = ['publishEvents', 'dedupe', competitionId, seasonId, fixtureId].join('/');
    const pubs_url = url.resolve(config.optaApiUrl, '/' + pubs_path);
    const pubEvents = R.flatten(R.of(publishes));
    request.post({url: pubs_url, json: true, body: publishes}, (err, res) => {
        if (err) {
            return callback(err);
        }
        if (res.statusCode !== 201) {
            return callback(`HTTP code ${res.statusCode} returned: ${JSON.stringify(res.body, null, 4)}`);
        }
        return callback (null, res.body);
    });
};

exports.fetchLandscapeConfig = (competitionId, seasonId, fixtureId, callback) => {
    const f9_path = ['feeds', 'F9', competitionId, seasonId, fixtureId].join('/');
    const f40_path = ['feeds', 'F40', competitionId, seasonId].join('/');
    const f9_url = url.resolve(config.optaApiUrl, '/' + f9_path);
    const f40_url = url.resolve(config.optaApiUrl, '/' + f40_path);

    const parseApiOutput = H.wrapCallback((response, callback) => {
        if (response.statusCode !== 200) {
            return callback (`HTTP code ${response.statusCode} returned: ${JSON.stringify(response.body, null, 4)}`);
        }
        return callback(null, response.body);
    });

    H([
        H.wrapCallback(request.get)({url: f9_url, json: true}).flatMap(parseApiOutput),
        H.wrapCallback(request.get)({url: f40_url, json: true}).flatMap(parseApiOutput)
    ])
        .parallel(2)
        .collect()
        .map((feeds) => {
            const F9 = feeds[0], F40 = feeds[1];
            return {
                teams: F9.competition.fixtures[0].teams.reduce((teams, team) => {
                    const f40Team = R.find((t) => {
                        return t.id === team.id;
                    }, F40.competition.teams);
                    return R.assocPath([team.id], R.merge(team, R.pick(['short_club_name','symid','players'],f40Team)), teams);
                }, {}),
                venue: R.path ( [ 'competition', 'fixtures', 0, 'venue' ], F9 )
            };
        })
        .errors(R.unary(callback))
        .apply(R.unary(R.partial(callback, [null])));
};

exports.events = function (competitionId, seasonId, fixtureId) {
    var events_path = ['events', competitionId, seasonId, fixtureId].join('/');
    var events_url = url.resolve(config.optaApiUrl, '/' + events_path);
    return eventStream(events_url);
};
